//------------------------------------------------------------------------------
/*
    This file is part of Bottle2D.

    Bottle2D is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Bottle2D is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Bottle2D.  If not, see <http://www.gnu.org/licenses/>.
*/
//------------------------------------------------------------------------------
#include "ClassWithProtectedRc.hpp"

#include <algorithm>

#include "MyException.hpp"
//------------------------------------------------------------------------------
namespace En
{
//------------------------------------------------------------------------------
void ClassWithProtectedRcBase::CheckResources()
{
    if(! AreResourcesOk())
    {
        MY_THROW("ClassWithProtectedRcBase::CheckResources() : Bad ressource !")
    }
}
//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
bool ClassWithProtectedRcSingle::AreResourcesOk()
{
    return m_ptr.IsValid();
}
//------------------------------------------------------------------------------
void ClassWithProtectedRcSingle::SetResource(ProtectedPtr<ProtectedClassBase> rcPtr)
{
    m_ptr = rcPtr;
}
//------------------------------------------------------------------------------
ProtectedPtr<ProtectedClassBase> ClassWithProtectedRcSingle::GetResource() const
{
    return m_ptr;
}
//------------------------------------------------------------------------------
void ClassWithProtectedRcSingle::ClearResource()
{
    m_ptr = NULL;
}
//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
bool ClassWithProtectedRcMultiple::AreResourcesOk()
{
    for(std::list<ProtectedPtr<ProtectedClassBase> >::const_iterator it = m_ptrList.begin(); it != m_ptrList.end(); it++)
    {
        if(! it->IsValid())
        {
            return false;
        }
    }

    return true;
}
//------------------------------------------------------------------------------
void ClassWithProtectedRcMultiple::AddResource(ProtectedPtr<ProtectedClassBase> rcPtr)
{
    m_ptrList.push_back(rcPtr);
}
//------------------------------------------------------------------------------
void ClassWithProtectedRcMultiple::RemoveResource(ProtectedPtr<ProtectedClassBase> rcPtrToRemove)
{
    std::list<ProtectedPtr<ProtectedClassBase> >::iterator it = find(m_ptrList.begin(), m_ptrList.end(), rcPtrToRemove);

    if(it != m_ptrList.end())
    {
        m_ptrList.erase(it);
    }
    else
    {
        MY_THROW("ClassWithProtectedRcMultiple::RemoveResource() : Resource to remove not found !")
    }
}
//------------------------------------------------------------------------------
void ClassWithProtectedRcMultiple::ClearResources()
{
    m_ptrList.clear();
}
//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
} // namespace En
//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
