
#include "LuaExpose.hpp"
//------------------------------------------------------------------------------
#include "Utils.hpp"
#include "ConfigManager.hpp"
#include "Event.hpp"
#include "PseudoEvent.hpp"
#include "Object.hpp"
#include "Drawable.hpp"
#include "PhysicObject.hpp"
#include "Group.hpp"
#include "Screen.hpp"
#include "Global.hpp"
#include "MyException.hpp"
#include "nbStrConvert.hpp"
#include "ParticleSource.hpp"
#include "PhCircle.hpp"
#include "PhLine.hpp"
#include "PhRect.hpp"
#include "PhShape.hpp"
#include "PhTriangle.hpp"
#include "PhysicWorld.hpp"
#include "PostFX.hpp"
#include "Shape.hpp"
#include "Sprite.hpp"
#include "Text.hpp"
#include "ScreenRenderer.hpp"
#include "TexturedArea.hpp"
#include "SoundManager.hpp"
//------------------------------------------------------------------------------
// Lua
#include <lua.hpp>
#include <luabind/luabind.hpp>
#include <luabind/operator.hpp>
//------------------------------------------------------------------------------
namespace En
{
//------------------------------------------------------------------------------
using namespace Global;
//------------------------------------------------------------------------------

void ExposeLua(lua_State *myLuaState)
{

    /////////////////////////////
    // Utils.hpp
    /////////////////////////////

    luabind::module(myLuaState)
    [
        luabind::class_<Vector3D>("Vector3D")
            .def(luabind::constructor<float , float , float >())
            .def(luabind::constructor<>())
            .def(luabind::self + luabind::other<const Vector3D& >())
            .def(luabind::self - luabind::other<const Vector3D& >())
            .def(luabind::self == luabind::other<const Vector3D& >())
            .def(luabind::self * luabind::other<float >())
            .def(luabind::self / luabind::other<float >())
            .def("GetSquareSum",(float(Vector3D::*)(void)) &Vector3D::GetSquareSum)
            .def("GetSize",(float(Vector3D::*)(void)) &Vector3D::GetSize)
            .def("GetDirection",(Vector3D(Vector3D::*)(void)) &Vector3D::GetDirection)
            .def("GetDotProduct",(float(Vector3D::*)(const Vector3D &)) &Vector3D::GetDotProduct)
            .def_readwrite("x", &Vector3D::x)
            .def_readwrite("y", &Vector3D::y)
            .def_readwrite("z", &Vector3D::z)
    ];

    luabind::module(myLuaState)
    [
        luabind::class_<Vector2D>("Vector2D")
            .def(luabind::constructor<float , float >())
            .def(luabind::constructor<>())
            .def(luabind::self + luabind::other<const Vector2D& >())
            .def(luabind::self - luabind::other<const Vector2D& >())
            .def(luabind::self == luabind::other<const Vector2D& >())
            .def(luabind::self * luabind::other<float >())
            .def(luabind::self / luabind::other<float >())
            .def("GetSquareSum",(float(Vector2D::*)(void)) &Vector2D::GetSquareSum)
            .def("GetSize",(float(Vector2D::*)(void)) &Vector2D::GetSize)
            .def("GetDirection",(Vector2D(Vector2D::*)(void)) &Vector2D::GetDirection)
            .def("GetDotProduct",(float(Vector2D::*)(const Vector2D &)) &Vector2D::GetDotProduct)
            .def("GetAngle",(float(Vector2D::*)(void)) &Vector2D::GetAngle)
            .def("GetDirectNormal",(Vector2D(Vector2D::*)(void)) &Vector2D::GetDirectNormal)
            .def("GetIndirectNormal",(Vector2D(Vector2D::*)(void)) &Vector2D::GetIndirectNormal)
            .def_readwrite("x", &Vector2D::x)
            .def_readwrite("y", &Vector2D::y)
    ];

    luabind::module(myLuaState)
    [
        luabind::class_<Color>("Color")
            .def(luabind::constructor<>())
            .def(luabind::constructor<char , char , char >())
            .def(luabind::constructor<char , char , char , char >())
            .def_readwrite("r", &Color::r)
            .def_readwrite("g", &Color::g)
            .def_readwrite("b", &Color::b)
            .def_readwrite("a", &Color::a)
    ];

    luabind::module(myLuaState)
    [
        luabind::class_<Texture>("Texture")
            .def(luabind::constructor<>())
            .def(luabind::constructor<const std::string &>())
            .def("GetWidth",(float(Texture::*)(void)) &Texture::GetWidth)
            .def("GetHeight",(float(Texture::*)(void)) &Texture::GetHeight)
            .def("GetSize",(Vector2D(Texture::*)(void)) &Texture::GetSize)
            .def("LoadFromFile",(void(Texture::*)(const std::string &)) &Texture::LoadFromFile)
    ];

    luabind::module(myLuaState)
    [
        luabind::class_<Sound>("Sound")
            .def(luabind::constructor<>())
            .def(luabind::constructor<const std::string &>())
            .def("LoadFromFile",(void(Sound::*)(const std::string &)) &Sound::LoadFromFile)
            .def("GetSampleRate",(int(Sound::*)(void)) &Sound::GetSampleRate)
            .def("GetChannelsCount",(int(Sound::*)(void)) &Sound::GetChannelsCount)
            .def("GetDuration",(float(Sound::*)(void)) &Sound::GetDuration)
    ];

    luabind::module(myLuaState)
    [
        luabind::class_<Music>("Music")
            .def(luabind::constructor<>())
            .def(luabind::constructor<const std::string &>())
            .def("LoadFromFile",(void(Music::*)(const std::string &)) &Music::LoadFromFile)
            .def("GetDuration",(float(Music::*)(void)) &Music::GetDuration)
            .def("GetChannelsCount",(int(Music::*)(void)) &Music::GetChannelsCount)
            .def("Play",(void(Music::*)(void)) &Music::Play)
            .def("Pause",(void(Music::*)(void)) &Music::Pause)
            .def("Stop",(void(Music::*)(void)) &Music::Stop)
            .def("SetLoop",(void(Music::*)(bool )) &Music::SetLoop)
            .def("SetPitch",(void(Music::*)(float )) &Music::SetPitch)
            .def("SetVolume",(void(Music::*)(float )) &Music::SetVolume)
            .def("GetLoop",(bool(Music::*)(void)) &Music::GetLoop)
            .def("GetPitch",(float(Music::*)(void)) &Music::GetPitch)
            .def("GetVolume",(float(Music::*)(void)) &Music::GetVolume)
            .def("GetPlayingOffset",(float(Music::*)(void)) &Music::GetPlayingOffset)
            .def("IsPlaying",(bool(Music::*)(void)) &Music::IsPlaying)
    ];

    luabind::module(myLuaState)
    [
        luabind::class_<Font>("Font")
            .def(luabind::constructor<>())
            .def(luabind::constructor<const std::string &>())
            .def("LoadFromFile",(void(Font::*)(const std::string &)) &Font::LoadFromFile)
    ];


    /////////////////////////////
    // ConfigManager.hpp
    /////////////////////////////

    luabind::module(myLuaState)
    [
        luabind::class_<Arguments>("Arguments")
            .def("GetCount",(int(Arguments::*)(void)) &Arguments::GetCount)
            .def("GetNext",(std::string(Arguments::*)(void)) &Arguments::GetNext)
            .def("Add",(void(Arguments::*)(const std::string &)) &Arguments::Add)
            .def("Clear",(void(Arguments::*)(void)) &Arguments::Clear)
    ];

    luabind::module(myLuaState)
    [
        luabind::class_<ModeInfo>("ModeInfo")
            .def_readwrite("fullName", &ModeInfo::fullName)
            .def_readwrite("idName", &ModeInfo::idName)
            .def_readwrite("auther", &ModeInfo::auther)
            .def_readwrite("path", &ModeInfo::path)
            .def_readwrite("description", &ModeInfo::description)
            .def_readwrite("version", &ModeInfo::version)
            .def_readwrite("screenshot", &ModeInfo::screenshot)
    ];

    luabind::module(myLuaState)
    [
        luabind::class_<ConfigNode>("ConfigNode")
            .def("GetString",(std::string(ConfigNode::*)(const std::string &)) &ConfigNode::GetString)
            .def("GetNumber",(float(ConfigNode::*)(const std::string &)) &ConfigNode::GetNumber)
            .def("GetValue",(luabind::object(ConfigNode::*)(const std::string &)) &ConfigNode::GetValue)
            .def("ChildExists",(bool(ConfigNode::*)(const std::string &)) &ConfigNode::ChildExists)
            .def("GetLuaTable",(luabind::object(ConfigNode::*)(void)) &ConfigNode::GetLuaTable)
            .def("SetValue",(void(ConfigNode::*)(const std::string &, luabind::object )) &ConfigNode::SetValue)
            .def("SetLuaTable",(void(ConfigNode::*)(luabind::object )) &ConfigNode::SetLuaTable)
            .def("AccessNode",(ConfigNode(ConfigNode::*)(const std::string &)) &ConfigNode::AccessNode)
            .def("Remove",(void(ConfigNode::*)(const std::string &)) &ConfigNode::Remove)
            .def("Clear",(void(ConfigNode::*)(void)) &ConfigNode::Clear)
            .def("IsDeleted",(bool(ConfigNode::*)(void)) &ConfigNode::IsDeleted)
    ];

    luabind::module(myLuaState)
    [
        luabind::class_<ConfigManager>("ConfigManager")
            .def("GetModeInfo",(ModeInfo(ConfigManager::*)(const std::string &)) &ConfigManager::GetModeInfo)
            .def("GetModeInfo",(ModeInfo(ConfigManager::*)(int )) &ConfigManager::GetModeInfo)
            .def("GetCurrentModeId",(std::string(ConfigManager::*)(void)) &ConfigManager::GetCurrentModeId)
            .def("GetRootConfigNode",(ConfigNode(ConfigManager::*)(void)) &ConfigManager::GetRootConfigNode)
            .def("GetModeCount",(int(ConfigManager::*)(void)) &ConfigManager::GetModeCount)
            .def("GetGlobalConfigNode",(ConfigNode(ConfigManager::*)(void)) &ConfigManager::GetGlobalConfigNode)
            .def("InitLocales",(void(ConfigManager::*)(const std::string &)) &ConfigManager::InitLocales)
    ];


    /////////////////////////////
    // Event.hpp
    /////////////////////////////

    luabind::module(myLuaState)
    [
        luabind::class_<EventHandlerBase, ProtectedPtr<EventHandlerBase> >("EventHandlerBase")
            .def("Destroy",(void(EventHandlerBase::*)(void)) &EventHandlerBase::Destroy)
            .def("ShouldBeDestroyed",(bool(EventHandlerBase::*)(void)) &EventHandlerBase::ShouldBeDestroyed)
            .def("SetUserData",(void(EventHandlerBase::*)(const std::string &, const luabind::object &)) &EventHandlerBase::SetUserData)
            .def("GetUserData",(luabind::object(EventHandlerBase::*)(const std::string &)) &EventHandlerBase::GetUserData)
            .def("UserDataExists",(bool(EventHandlerBase::*)(const std::string &)) &EventHandlerBase::UserDataExists)
            .def("GetLuaFunction",(luabind::object(EventHandlerBase::*)(void)) &EventHandlerBase::GetLuaFunction)
            .def("SetLuaFunction",(void(EventHandlerBase::*)(luabind::object )) &EventHandlerBase::SetLuaFunction)
            .def(luabind::self == luabind::other<const EventHandlerBase& >())
    ];


    /////////////////////////////
    // PseudoEvent.hpp
    /////////////////////////////

    luabind::module(myLuaState)
    [
        luabind::class_<Timer, EventHandlerBase, ProtectedPtr<EventHandlerBase> >("Timer")
            .property("interval", &Timer:: GetInterval, &Timer:: SetInterval)
            .def("GetInterval",(float(Timer::*)(void)) &Timer::GetInterval)
            .def("SetInterval",(void(Timer::*)(float )) &Timer::SetInterval)
            .def("GetElapsedTime",(float(Timer::*)(void)) &Timer::GetElapsedTime)
            .def("SetElapsedTime",(void(Timer::*)(float )) &Timer::SetElapsedTime)
            .def("Start",(void(Timer::*)(void)) &Timer::Start)
            .def("Stop",(void(Timer::*)(void)) &Timer::Stop)
            .def("IsRunning",(bool(Timer::*)(void)) &Timer::IsRunning)
    ];

    luabind::module(myLuaState)
    [
        luabind::class_<Chronometer, EventHandlerBase, ProtectedPtr<EventHandlerBase> >("Chronometer")
            .property("totalTime", &Chronometer:: GetTotalTime, &Chronometer:: SetTotalTime)
            .def("GetTotalTime",(float(Chronometer::*)(void)) &Chronometer::GetTotalTime)
            .def("SetTotalTime",(void(Chronometer::*)(float )) &Chronometer::SetTotalTime)
            .property("ratio", &Chronometer:: GetRatio, &Chronometer:: SetRatio)
            .def("GetRatio",(float(Chronometer::*)(void)) &Chronometer::GetRatio)
            .def("SetRatio",(void(Chronometer::*)(float )) &Chronometer::SetRatio)
            .def("Restart",(void(Chronometer::*)(void)) &Chronometer::Restart)
            .def("Start",(void(Chronometer::*)(void)) &Chronometer::Start)
            .def("Stop",(void(Chronometer::*)(void)) &Chronometer::Stop)
            .def("IsRunning",(bool(Chronometer::*)(void)) &Chronometer::IsRunning)
    ];

    luabind::module(myLuaState)
    [
        luabind::class_<MouseZoneEvent, EventHandlerBase, ProtectedPtr<EventHandlerBase> >("MouseZoneEvent")
            .property("zone", &MouseZoneEvent:: GetZone, &MouseZoneEvent:: SetZone)
            .def("GetZone",(ProtectedPtr<PhysicObject>(MouseZoneEvent::*)(void)) &MouseZoneEvent::GetZone)
            .def("SetZone",(void(MouseZoneEvent::*)(ProtectedPtr<PhysicObject> )) &MouseZoneEvent::SetZone)
            .def("IsInside",(bool(MouseZoneEvent::*)(void)) &MouseZoneEvent::IsInside)
    ];


    /////////////////////////////
    // Object.hpp
    /////////////////////////////

    luabind::module(myLuaState)
    [
        luabind::class_<Object, ProtectedPtr<Object> >("Object")
            .def(luabind::self == luabind::other<const Object& >())
            .def("Clone",(luabind::object(Object::*)(void)) &Object::Clone)
            .def("CloneToGroup",(luabind::object(Object::*)(Group *)) &Object::CloneToGroup)
            .property("position", &Object:: GetPosition, &Object:: SetPosition)
            .def("GetPosition",(Vector2D(Object::*)(void)) &Object::GetPosition)
            .def("SetPosition",(void(Object::*)(const Vector2D &)) &Object::SetPosition)
            .property("rotation", &Object:: GetRotation, &Object:: SetRotation)
            .def("GetRotation",(float(Object::*)(void)) &Object::GetRotation)
            .def("SetRotation",(void(Object::*)(float )) &Object::SetRotation)
            .property("x", &Object:: GetPositionX, &Object:: SetPositionX)
            .property("y", &Object:: GetPositionY, &Object:: SetPositionY)
            .def("GetPositionX",(float(Object::*)(void)) &Object::GetPositionX)
            .def("GetPositionY",(float(Object::*)(void)) &Object::GetPositionY)
            .def("SetPositionX",(void(Object::*)(float )) &Object::SetPositionX)
            .def("SetPositionY",(void(Object::*)(float )) &Object::SetPositionY)
            .property("globalPosition", &Object:: GetGlobalPosition, &Object:: SetGlobalPosition)
            .def("GetGlobalPosition",(Vector2D(Object::*)(void)) &Object::GetGlobalPosition)
            .def("SetGlobalPosition",(void(Object::*)(const Vector2D &)) &Object::SetGlobalPosition)
            .property("rotation", &Object:: GetGlobalRotation, &Object:: SetGlobalRotation)
            .def("GetGlobalRotation",(float(Object::*)(void)) &Object::GetGlobalRotation)
            .def("SetGlobalRotation",(void(Object::*)(float )) &Object::SetGlobalRotation)
            .def("Rotate",(void(Object::*)(float )) &Object::Rotate)
            .def("Move",(void(Object::*)(const Vector2D &)) &Object::Move)
            .property("name", &Object:: GetName, &Object:: SetName)
            .def("GetName",(std::string(Object::*)(void)) &Object::GetName)
            .def("SetName",(void(Object::*)(const std::string &)) &Object::SetName)
            .property("speed", &Object:: GetSpeed, &Object:: SetSpeed)
            .def("SetSpeed",(void(Object::*)(const Vector2D &)) &Object::SetSpeed)
            .def("GetSpeed",(Vector2D(Object::*)(void)) &Object::GetSpeed)
            .property("angularVelocity", &Object:: GetAngularVelocity, &Object:: SetAngularVelocity)
            .def("SetAngularVelocity",(void(Object::*)(float )) &Object::SetAngularVelocity)
            .def("GetAngularVelocity",(float(Object::*)(void)) &Object::GetAngularVelocity)
            .def("GetType",(int(Object::*)(void)) &Object::GetType)
            .def("GetUniqueId",(long(Object::*)(void)) &Object::GetUniqueId)
            .def("ChangeParent",(void(Object::*)(Group *)) &Object::ChangeParent)
            .def("Destroy",(void(Object::*)(void)) &Object::Destroy)
            .def("GetParent",(luabind::object(Object::*)(void)) &Object::GetParent)
            .def("SetUserData",(void(Object::*)(const std::string &, const luabind::object &)) &Object::SetUserData)
            .def("GetUserData",(luabind::object(Object::*)(const std::string &)) &Object::GetUserData)
            .def("UserDataExists",(bool(Object::*)(const std::string &)) &Object::UserDataExists)
            .def("GetParentScreen",(ProtectedPtr<Screen>(Object::*)(void)) &Object::GetParentScreen)
    ];


    /////////////////////////////
    // Drawable.hpp
    /////////////////////////////

    luabind::module(myLuaState)
    [
        luabind::class_<Drawable, Object, ProtectedPtr<Object> >("Drawable")
            .property("zPos", &Drawable:: GetZpos, &Drawable:: SetZpos)
            .def("GetZpos",(int(Drawable::*)(void)) &Drawable::GetZpos)
            .def("SetZpos",(void(Drawable::*)(int )) &Drawable::SetZpos)
            .def("GetLowZPos",(int(Drawable::*)(void)) &Drawable::GetLowZPos)
            .property("visible", &Drawable:: IsVisible, &Drawable:: Show)
            .def("Show",(void(Drawable::*)(bool )) &Drawable::Show)
            .def("Hide",(void(Drawable::*)(void)) &Drawable::Hide)
            .def("IsVisible",(bool(Drawable::*)(void)) &Drawable::IsVisible)
            .def("IsUpperThan",(bool(Drawable::*)(const Drawable *)) &Drawable::IsUpperThan)
    ];


    /////////////////////////////
    // PhysicObject.hpp
    /////////////////////////////

    luabind::module(myLuaState)
    [
        luabind::class_<PhysicObject, Object, ProtectedPtr<Object> >("PhysicObject")
            .def("IsPointInside",(bool(PhysicObject::*)(const Vector2D &)) &PhysicObject::IsPointInside)
            .property("eventFunction", &PhysicObject:: GetEventFunction, &PhysicObject:: SetEventFunction)
            .def("SetEventFunction",(void(PhysicObject::*)(const luabind::object &)) &PhysicObject::SetEventFunction)
            .def("GetEventFunction",(luabind::object(PhysicObject::*)(void)) &PhysicObject::GetEventFunction)
            .def("GetPhysicGroup",(int(PhysicObject::*)(void)) &PhysicObject::GetPhysicGroup)
            .def("IsIntersect",(bool(PhysicObject::*)(const PhysicObject &)) &PhysicObject::IsIntersect)
            .def("IsIntersectWithGroup",(luabind::object(PhysicObject::*)(int )) &PhysicObject::IsIntersectWithGroup)
    ];

    luabind::module(myLuaState)
    [
        luabind::class_<CollisionInfo>("CollisionInfo")
            .def_readwrite("phObj", &CollisionInfo::phObj)
            .def_readwrite("point", &CollisionInfo::point)
            .def_readwrite("penetrationVector", &CollisionInfo::penetrationVector)
    ];


    /////////////////////////////
    // Group.hpp
    /////////////////////////////

    luabind::module(myLuaState)
    [
        luabind::class_<Group, Drawable, ProtectedPtr<Object> >("Group")
            .property("scale", &Group:: GetScale, &Group:: SetScale)
            .def("SetScale",(void(Group::*)(const Vector2D &)) &Group::SetScale)
            .def("GetScale",(Vector2D(Group::*)(void)) &Group::GetScale)
            .def("GetChildCount",(int(Group::*)(void)) &Group::GetChildCount)
            .def("GetPhysicChild",(luabind::object(Group::*)(int )) &Group::GetPhysicChild)
            .def("GetPhysicChildCount",(int(Group::*)(void)) &Group::GetPhysicChildCount)
            .def("GetDrawableChild",(luabind::object(Group::*)(int )) &Group::GetDrawableChild)
            .def("GetDrawableChildCount",(int(Group::*)(void)) &Group::GetDrawableChildCount)
            .def("DestroyChildren",(void(Group::*)(void)) &Group::DestroyChildren)
            .def("EnableCut",(void(Group::*)(bool )) &Group::EnableCut)
            .def("IsCut",(bool(Group::*)(void)) &Group::IsCut)
            .def("SetCutRect",(void(Group::*)(const Vector2D &, const Vector2D &)) &Group::SetCutRect)
            .def("GetCutRectPosition",(Vector2D(Group::*)(void)) &Group::GetCutRectPosition)
            .def("GetCutRectSize",(Vector2D(Group::*)(void)) &Group::GetCutRectSize)
            .def("GetChildTable",(luabind::object(Group::*)(void)) &Group::GetChildTable)
            .def("FindByIndex",(luabind::object(Group::*)(int )) &Group::FindByIndex)
            .def("FindByName",(luabind::object(Group::*)(const std::string &)) &Group::FindByName)
            .def("FindByNamePhLine",(ProtectedPtr<PhLine>(Group::*)(const std::string &)) &Group::FindByNamePhLine)
            .def("FindByNamePhRect",(ProtectedPtr<PhRect>(Group::*)(const std::string &)) &Group::FindByNamePhRect)
            .def("FindByNamePhTriangle",(ProtectedPtr<PhTriangle>(Group::*)(const std::string &)) &Group::FindByNamePhTriangle)
            .def("FindByNamePhCircle",(ProtectedPtr<PhCircle>(Group::*)(const std::string &)) &Group::FindByNamePhCircle)
            .def("FindByNamePhShape",(ProtectedPtr<PhShape>(Group::*)(const std::string &)) &Group::FindByNamePhShape)
            .def("FindByNameGroup",(ProtectedPtr<Group>(Group::*)(const std::string &)) &Group::FindByNameGroup)
            .def("FindByNameParticleSource",(ProtectedPtr<ParticleSource>(Group::*)(const std::string &)) &Group::FindByNameParticleSource)
            .def("FindByNameShape",(ProtectedPtr<Shape>(Group::*)(const std::string &)) &Group::FindByNameShape)
            .def("FindByNameSprite",(ProtectedPtr<Sprite>(Group::*)(const std::string &)) &Group::FindByNameSprite)
            .def("FindByNameText",(ProtectedPtr<Text>(Group::*)(const std::string &)) &Group::FindByNameText)
            .def("FindByNamePostFX",(ProtectedPtr<PostFX>(Group::*)(const std::string &)) &Group::FindByNamePostFX)
            .def("FindByNameScreen",(ProtectedPtr<Screen>(Group::*)(const std::string &)) &Group::FindByNameScreen)
            .def("FindByNameScreenRenderer",(ProtectedPtr<ScreenRenderer>(Group::*)(const std::string &)) &Group::FindByNameScreenRenderer)
            .def("FindByNameTexturedArea",(ProtectedPtr<TexturedArea>(Group::*)(const std::string &)) &Group::FindByNameTexturedArea)
            .def("CreatePhLine",(ProtectedPtr<PhLine>(Group::*)(int , const std::string &, const Vector2D &, const Vector2D &)) &Group::CreatePhLine)
            .def("CreatePhRect",(ProtectedPtr<PhRect>(Group::*)(int , const std::string &, const Vector2D &)) &Group::CreatePhRect)
            .def("CreatePhRect",(ProtectedPtr<PhRect>(Group::*)(int , const std::string &, const Vector2D &, const Vector2D &)) &Group::CreatePhRect)
            .def("CreatePhTriangle",(ProtectedPtr<PhTriangle>(Group::*)(int , const std::string &, const Vector2D &, const Vector2D &, const Vector2D &)) &Group::CreatePhTriangle)
            .def("CreatePhShape",(ProtectedPtr<PhShape>(Group::*)(int , const std::string &)) &Group::CreatePhShape)
            .def("CreatePhCircle",(ProtectedPtr<PhCircle>(Group::*)(int , const std::string &, const Vector2D &, float )) &Group::CreatePhCircle)
            .def("CreateGroup",(ProtectedPtr<Group>(Group::*)(const std::string &)) &Group::CreateGroup)
            .def("CreateGroup",(ProtectedPtr<Group>(Group::*)(const std::string &, const Vector2D &)) &Group::CreateGroup)
            .def("CreateParticleSource",(ProtectedPtr<ParticleSource>(Group::*)(const std::string &, const ParticleModel &, int )) &Group::CreateParticleSource)
            .def("CreateShape",(ProtectedPtr<Shape>(Group::*)(const std::string &)) &Group::CreateShape)
            .def("CreateShapeLine",(ProtectedPtr<Shape>(Group::*)(const std::string &, const Vector2D& , const Vector2D& , float , const Color& , float , const Color& )) &Group::CreateShapeLine)
            .def("CreateShapeRectangle",(ProtectedPtr<Shape>(Group::*)(const std::string &, const Vector2D& , const Vector2D& , const Color& , float , const Color& )) &Group::CreateShapeRectangle)
            .def("CreateShapeRectangleSize",(ProtectedPtr<Shape>(Group::*)(const std::string &, const Vector2D& , const Vector2D& , const Color& , float , const Color& )) &Group::CreateShapeRectangleSize)
            .def("CreateShapeCircle",(ProtectedPtr<Shape>(Group::*)(const std::string &, const Vector2D& , float , const Color& , float , const Color& )) &Group::CreateShapeCircle)
            .def("CreateSprite",(ProtectedPtr<Sprite>(Group::*)(const std::string &)) &Group::CreateSprite)
            .def("CreateSprite",(ProtectedPtr<Sprite>(Group::*)(const std::string &, Texture &, const Vector2D &)) &Group::CreateSprite)
            .def("CreateText",(ProtectedPtr<Text>(Group::*)(const std::string &, const std::string &)) &Group::CreateText)
            .def("CreateText",(ProtectedPtr<Text>(Group::*)(const std::string &, const std::string &, int , const Color &, const Vector2D &)) &Group::CreateText)
            .def("CreatePostFX",(ProtectedPtr<PostFX>(Group::*)(const std::string &)) &Group::CreatePostFX)
            .def("CreateScreenRenderer",(ProtectedPtr<ScreenRenderer>(Group::*)(const std::string &)) &Group::CreateScreenRenderer)
            .def("CreateTexturedArea",(ProtectedPtr<TexturedArea>(Group::*)(const std::string &, Texture &, const Vector2D &, const Vector2D &)) &Group::CreateTexturedArea)
            .def("SetPreDestroyFunction",(void(Group::*)(const luabind::object &)) &Group::SetPreDestroyFunction)
            .def("GetPreDestroyFunction",(luabind::object(Group::*)(void)) &Group::GetPreDestroyFunction)
    ];


    /////////////////////////////
    // Screen.hpp
    /////////////////////////////

    luabind::module(myLuaState)
    [
        luabind::class_<ScreenConfig>("ScreenConfig")
            .def(luabind::constructor<>())
            .def(luabind::constructor<const Vector2D &>())
            .def("ChangeSize",(void(ScreenConfig::*)(const Vector2D &)) &ScreenConfig::ChangeSize)
            .def("CameraReset",(void(ScreenConfig::*)(void)) &ScreenConfig::CameraReset)
            .property("forcedRatio", &ScreenConfig:: GetForcedRatio, &ScreenConfig:: SetForcedRatio)
            .def("SetForcedRatio",(void(ScreenConfig::*)(float )) &ScreenConfig::SetForcedRatio)
            .def("GetForcedRatio",(float(ScreenConfig::*)(void)) &ScreenConfig::GetForcedRatio)
            .property("isFixedRatio", &ScreenConfig:: IsFixedRatio, &ScreenConfig:: SetFixedRatio)
            .def("SetFixedRatio",(void(ScreenConfig::*)(bool )) &ScreenConfig::SetFixedRatio)
            .def("IsFixedRatio",(bool(ScreenConfig::*)(void)) &ScreenConfig::IsFixedRatio)
            .property("screenSize", &ScreenConfig:: GetScreenSize, &ScreenConfig:: SetScreenSize)
            .def("SetScreenSize",(void(ScreenConfig::*)(const Vector2D &)) &ScreenConfig::SetScreenSize)
            .def("GetScreenSize",(Vector2D(ScreenConfig::*)(void)) &ScreenConfig::GetScreenSize)
            .property("cameraTopLeft", &ScreenConfig:: GetCameraTopLeft, &ScreenConfig:: SetCameraTopLeft)
            .def("SetCameraTopLeft",(void(ScreenConfig::*)(const Vector2D &)) &ScreenConfig::SetCameraTopLeft)
            .def("GetCameraTopLeft",(Vector2D(ScreenConfig::*)(void)) &ScreenConfig::GetCameraTopLeft)
            .property("cameraBottomRight", &ScreenConfig:: GetCameraBottomRight, &ScreenConfig:: SetCameraBottomRight)
            .def("SetCameraBottomRight",(void(ScreenConfig::*)(const Vector2D &)) &ScreenConfig::SetCameraBottomRight)
            .def("GetCameraBottomRight",(Vector2D(ScreenConfig::*)(void)) &ScreenConfig::GetCameraBottomRight)
            .def("CameraMove",(void(ScreenConfig::*)(const Vector2D &)) &ScreenConfig::CameraMove)
            .def("CameraZoom",(void(ScreenConfig::*)(float )) &ScreenConfig::CameraZoom)
            .def("CameraZoom",(void(ScreenConfig::*)(Vector2D )) &ScreenConfig::CameraZoom)
            .def("CameraSetZoom",(void(ScreenConfig::*)(float )) &ScreenConfig::CameraSetZoom)
            .def("CameraSetZoom",(void(ScreenConfig::*)(Vector2D )) &ScreenConfig::CameraSetZoom)
            .def("CameraGetZoom",(Vector2D(ScreenConfig::*)(void)) &ScreenConfig::CameraGetZoom)
            .property("cameraCenter", &ScreenConfig:: CameraGetCenter, &ScreenConfig:: CameraSetCenter)
            .def("CameraSetCenter",(void(ScreenConfig::*)(const Vector2D &)) &ScreenConfig::CameraSetCenter)
            .def("CameraGetCenter",(Vector2D(ScreenConfig::*)(void)) &ScreenConfig::CameraGetCenter)
            .property("cameraHalfSize", &ScreenConfig:: CameraGetHalfSize, &ScreenConfig:: CameraSetHalfSize)
            .def("CameraSetHalfSize",(void(ScreenConfig::*)(const Vector2D &)) &ScreenConfig::CameraSetHalfSize)
            .def("CameraGetHalfSize",(Vector2D(ScreenConfig::*)(void)) &ScreenConfig::CameraGetHalfSize)
            .property("cameraSize", &ScreenConfig:: CameraGetSize, &ScreenConfig:: CameraSetSize)
            .def("CameraSetSize",(void(ScreenConfig::*)(const Vector2D &)) &ScreenConfig::CameraSetSize)
            .def("CameraGetSize",(Vector2D(ScreenConfig::*)(void)) &ScreenConfig::CameraGetSize)
    ];

    luabind::module(myLuaState)
    [
        luabind::class_<Screen, Group, ProtectedPtr<Object> >("Screen")
            .def("RegisterButtonEvent",(ProtectedPtr<EventHandlerBase>(Screen::*)(const luabind::object &, const std::string &, bool )) &Screen::RegisterButtonEvent)
            .def("RegisterButtonInZoneEvent",(ProtectedPtr<EventHandlerBase>(Screen::*)(const luabind::object &, MouseZoneEvent *, const std::string &, bool )) &Screen::RegisterButtonInZoneEvent)
            .def("RegisterMouseMoveEvent",(ProtectedPtr<EventHandlerBase>(Screen::*)(const luabind::object &)) &Screen::RegisterMouseMoveEvent)
            .def("RegisterTimer",(ProtectedPtr<Timer>(Screen::*)(const luabind::object &, float , bool )) &Screen::RegisterTimer)
            .def("RegisterChronometer",(ProtectedPtr<Chronometer>(Screen::*)(const luabind::object &, float , bool )) &Screen::RegisterChronometer)
            .def("RegisterMouseZoneEvent",(ProtectedPtr<MouseZoneEvent>(Screen::*)(const luabind::object &, PhysicObject *)) &Screen::RegisterMouseZoneEvent)
            .def("RegisterFrameEvent",(ProtectedPtr<EventHandlerBase>(Screen::*)(const luabind::object &)) &Screen::RegisterFrameEvent)
            .def("SetMouseRelative",(void(Screen::*)(bool )) &Screen::SetMouseRelative)
            .def("IsMouseRelative",(bool(Screen::*)(void)) &Screen::IsMouseRelative)
            .def("ShowCursor",(void(Screen::*)(bool )) &Screen::ShowCursor)
            .def("IsCursorVisible",(bool(Screen::*)(void)) &Screen::IsCursorVisible)
            .def("GetEventCount",(int(Screen::*)(void)) &Screen::GetEventCount)
            .property("renderScreen", &Screen:: GetRenderScreen, &Screen:: SetRenderScreen)
            .def("SetRenderScreen",(void(Screen::*)(luabind::object )) &Screen::SetRenderScreen)
            .def("GetRenderScreen",(luabind::object(Screen::*)(void)) &Screen::GetRenderScreen)
            .property("timeSpeed", &Screen:: GetTimeSpeed, &Screen:: SetTimeSpeed)
            .def("SetTimeSpeed",(void(Screen::*)(float )) &Screen::SetTimeSpeed)
            .def("GetTimeSpeed",(float(Screen::*)(void)) &Screen::GetTimeSpeed)
            .def("AddPhysicGroup",(void(Screen::*)(int , int )) &Screen::AddPhysicGroup)
            .property("backgroundColor", &Screen:: GetBackgroundColor, &Screen:: SetBackgroundColor)
            .def("GetBackgroundColor",(Color(Screen::*)(void)) &Screen::GetBackgroundColor)
            .def("SetBackgroundColor",(void(Screen::*)(const Color &)) &Screen::SetBackgroundColor)
            .property("screenConfig", &Screen:: GetScreenConfig, &Screen:: SetScreenConfig)
            .def("GetScreenConfig",(ScreenConfig(Screen::*)(void)) &Screen::GetScreenConfig)
            .def("SetScreenConfig",(void(Screen::*)(const ScreenConfig &)) &Screen::SetScreenConfig)
            .def("GetUnusedPhGroup",(int(Screen::*)(void)) &Screen::GetUnusedPhGroup)
            .def("PlaySound",(ProtectedPtr<SoundPlayer>(Screen::*)(Sound &)) &Screen::PlaySound)
            .def("StopAllSounds",(void(Screen::*)(void)) &Screen::StopAllSounds)
            .def("IsCurrentRunningScreen",(bool(Screen::*)(void)) &Screen::IsCurrentRunningScreen)
    ];


    /////////////////////////////
    // Global.hpp
    /////////////////////////////

    luabind::module(myLuaState)
    [
        luabind::def("ChangeScreen", (void(*)(const std::string &)) &ChangeScreen)
        ,luabind::def("ChangeScreen", (void(*)(ProtectedPtr<Screen> )) &ChangeScreen)
        ,luabind::def("CreateScreen", (ProtectedPtr<Screen>(*)(const std::string &)) &CreateScreen)
        ,luabind::def("CreateScreen", (ProtectedPtr<Screen>(*)(const std::string &, const ScreenConfig &)) &CreateScreen)
        ,luabind::def("GetCommonDataPath", (std::string(*)(void)) &GetCommonDataPath)
        ,luabind::def("GetModePath", (std::string(*)(void)) &GetModePath)
        ,luabind::def("GetEngineVersion", (std::string(*)(void)) &GetEngineVersion)
        ,luabind::def("RequestRecreateRenderWindow", (void(*)(void)) &RequestRecreateRenderWindow)
        ,luabind::def("GetFrameTime", (float(*)(void)) &GetFrameTime)
        ,luabind::def("Quit", (void(*)(void)) &Quit)
        ,luabind::def("ConvertRatioCosAccelerationDeceleration", (float(*)(float )) &ConvertRatioCosAccelerationDeceleration)
        ,luabind::def("ConvertRatioAcceleration", (float(*)(float , float )) &ConvertRatioAcceleration)
        ,luabind::def("ConvertRatioDeceleration", (float(*)(float , float )) &ConvertRatioDeceleration)
        ,luabind::def("GetMousePosition", (Vector2D(*)(void)) &GetMousePosition)
        ,luabind::def("GetModeConf", (ConfigNode(*)(void)) &GetModeConf)
        ,luabind::def("GetModeInfo", (ModeInfo(*)(void)) &GetModeInfo)
        ,luabind::def("LoadMode", (void(*)(const std::string &)) &LoadMode)
        ,luabind::def("GetConfigManager", (ConfigManager&(*)(void)) &GetConfigManager)
        ,luabind::def("ForceFrame", (void(*)(void)) &ForceFrame)
        ,luabind::def("ShowConsole", (void(*)(bool )) &ShowConsole)
        ,luabind::def("IsConsoleVisible", (bool(*)(void)) &IsConsoleVisible)
        ,luabind::def("IsKeyPressed", (bool(*)(const std::string &)) &IsKeyPressed)
        ,luabind::def("GetRealWindowSize", (Vector2D(*)(void)) &GetRealWindowSize)
        ,luabind::def("SetShowPhysicObjects", (void(*)(bool )) &SetShowPhysicObjects)
        ,luabind::def("IsShowPhysicObjects", (bool(*)(void)) &IsShowPhysicObjects)
        ,luabind::def("_", (std::string(*)(const std::string &)) &_)
        ,luabind::def("GetTypeName", (std::string(*)(int )) &GetTypeName)
        ,luabind::def("GetChildTree", (std::string(*)(void)) &GetChildTree)
        ,luabind::def("GetAvaibleVideoModes", (luabind::object(*)(void)) &GetAvaibleVideoModes)
        ,luabind::def("GetCmdArguments", (luabind::object(*)(void)) &GetCmdArguments)
        ,luabind::def("GetGlobalSoundManager", (SoundManager*(*)(void)) &GetGlobalSoundManager)
        ,luabind::def("GetCurrentScreen", (ProtectedPtr<Screen>(*)(void)) &GetCurrentScreen)
        ,luabind::def("GetScreenList", (luabind::object(*)(void)) &GetScreenList)
        ,luabind::def("GetUniqueFrameId", (int(*)(void)) &GetUniqueFrameId)
    ];


    /////////////////////////////
    // MyException.hpp
    /////////////////////////////


    /////////////////////////////
    // nbStrConvert.hpp
    /////////////////////////////


    /////////////////////////////
    // ParticleSource.hpp
    /////////////////////////////

    luabind::module(myLuaState)
    [
        luabind::class_<ParticleModel>("ParticleModel")
            .def(luabind::constructor<>())
            .def(luabind::constructor<float , float >())
            .def("AddFixedParam",(void(ParticleModel::*)(const std::string &, float )) &ParticleModel::AddFixedParam)
            .def("AddMutableParam",(void(ParticleModel::*)(const std::string &, float , float )) &ParticleModel::AddMutableParam)
            .def("AddRandomParam",(void(ParticleModel::*)(const std::string &, float , float )) &ParticleModel::AddRandomParam)
            .def("AddMutableRandomParam",(void(ParticleModel::*)(const std::string &, float , float , float , float )) &ParticleModel::AddMutableRandomParam)
            .def("SetLifeTime",(void(ParticleModel::*)(float , float )) &ParticleModel::SetLifeTime)
    ];

    luabind::module(myLuaState)
    [
        luabind::class_<ParticleEmitter>("ParticleEmitter")
            .def(luabind::constructor<>())
            .def("SetAngles",(void(ParticleEmitter::*)(float , float )) &ParticleEmitter::SetAngles)
            .def("SetDirection",(void(ParticleEmitter::*)(const Vector2D &)) &ParticleEmitter::SetDirection)
            .def("MakeLine",(void(ParticleEmitter::*)(const Vector2D &, const Vector2D &)) &ParticleEmitter::MakeLine)
            .def("MakeDisk",(void(ParticleEmitter::*)(const Vector2D &, float )) &ParticleEmitter::MakeDisk)
            .def("MakeCircle",(void(ParticleEmitter::*)(const Vector2D &, float )) &ParticleEmitter::MakeCircle)
            .def("MakePoint",(void(ParticleEmitter::*)(const Vector2D &)) &ParticleEmitter::MakePoint)
    ];

    luabind::module(myLuaState)
    [
        luabind::class_<ParticleSource, Drawable, ProtectedPtr<Object> >("ParticleSource")
            .property("flow", &ParticleSource:: GetFlow, &ParticleSource:: SetFlow)
            .def("SetFlow",(void(ParticleSource::*)(int )) &ParticleSource::SetFlow)
            .def("GetFlow",(int(ParticleSource::*)(void)) &ParticleSource::GetFlow)
            .property("tank", &ParticleSource:: GetTank, &ParticleSource:: SetTank)
            .def("SetTank",(void(ParticleSource::*)(int )) &ParticleSource::SetTank)
            .def("GetTank",(int(ParticleSource::*)(void)) &ParticleSource::GetTank)
            .property("gravity", &ParticleSource:: GetGravity, &ParticleSource:: SetGravity)
            .def("SetGravity",(void(ParticleSource::*)(const Vector2D &)) &ParticleSource::SetGravity)
            .def("GetGravity",(Vector2D(ParticleSource::*)(void)) &ParticleSource::GetGravity)
            .property("friction", &ParticleSource:: GetGravity, &ParticleSource:: GetFriction)
            .def("SetFriction",(void(ParticleSource::*)(float )) &ParticleSource::SetFriction)
            .def("GetFriction",(float(ParticleSource::*)(void)) &ParticleSource::GetFriction)
            .def("SetForce",(void(ParticleSource::*)(float , float )) &ParticleSource::SetForce)
            .property("minForce", &ParticleSource:: GetMinForce, &ParticleSource:: SetMinForce)
            .def("GetMinForce",(float(ParticleSource::*)(void)) &ParticleSource::GetMinForce)
            .def("SetMinForce",(void(ParticleSource::*)(float )) &ParticleSource::SetMinForce)
            .property("maxForce", &ParticleSource:: GetMaxForce, &ParticleSource:: SetMaxForce)
            .def("GetMaxForce",(float(ParticleSource::*)(void)) &ParticleSource::GetMaxForce)
            .def("SetMaxForce",(void(ParticleSource::*)(float )) &ParticleSource::SetMaxForce)
            .def("SetEmitterAngle",(void(ParticleSource::*)(float )) &ParticleSource::SetEmitterAngle)
            .def("GetEmitterAngle",(float(ParticleSource::*)(void)) &ParticleSource::GetEmitterAngle)
            .def("SetEmitterDirection",(void(ParticleSource::*)(const Vector2D &)) &ParticleSource::SetEmitterDirection)
            .def("GetEmitterDirection",(Vector2D(ParticleSource::*)(void)) &ParticleSource::GetEmitterDirection)
            .def("SetEmitter",(void(ParticleSource::*)(const ParticleEmitter &)) &ParticleSource::SetEmitter)
            .def("MakeRenderLines",(void(ParticleSource::*)(float , float )) &ParticleSource::MakeRenderLines)
            .def("MakeRenderPoints",(void(ParticleSource::*)(float )) &ParticleSource::MakeRenderPoints)
            .def("MakeRenderImages",(void(ParticleSource::*)(Texture &, const Vector2D &, bool )) &ParticleSource::MakeRenderImages)
            .property("autoDestroy", &ParticleSource:: IsAutoDestroy, &ParticleSource:: SetAutoDestroy)
            .def("SetAutoDestroy",(void(ParticleSource::*)(bool )) &ParticleSource::SetAutoDestroy)
            .def("IsAutoDestroy",(bool(ParticleSource::*)(void)) &ParticleSource::IsAutoDestroy)
            .def("ComputeParticles",(void(ParticleSource::*)(float )) &ParticleSource::ComputeParticles)
            .property("onTankEmptyEventFunction", &ParticleSource:: GetOnTankEmptyEventFunction, &ParticleSource:: SetOnTankEmptyEventFunction)
            .def("SetOnTankEmptyEventFunction",(void(ParticleSource::*)(const luabind::object &)) &ParticleSource::SetOnTankEmptyEventFunction)
            .def("GetOnTankEmptyEventFunction",(luabind::object(ParticleSource::*)(void)) &ParticleSource::GetOnTankEmptyEventFunction)
    ];


    /////////////////////////////
    // PhCircle.hpp
    /////////////////////////////

    luabind::module(myLuaState)
    [
        luabind::class_<PhCircle, PhysicObject, ProtectedPtr<Object> >("PhCircle")
            .property("radius", &PhCircle:: GetRadius, &PhCircle:: SetRadius)
            .def("SetRadius",(void(PhCircle::*)(float )) &PhCircle::SetRadius)
            .def("GetRadius",(float(PhCircle::*)(void)) &PhCircle::GetRadius)
    ];


    /////////////////////////////
    // PhLine.hpp
    /////////////////////////////

    luabind::module(myLuaState)
    [
        luabind::class_<PhLine, PhysicObject, ProtectedPtr<Object> >("PhLine")
            .property("vector", &PhLine:: GetVector, &PhLine:: SetVector)
            .def("SetVector",(void(PhLine::*)(const Vector2D &)) &PhLine::SetVector)
            .def("GetVector",(Vector2D(PhLine::*)(void)) &PhLine::GetVector)
            .property("pt2", &PhLine:: GetPt2, &PhLine:: SetPt2)
            .def("SetPt2",(void(PhLine::*)(const Vector2D &)) &PhLine::SetPt2)
            .def("GetPt2",(Vector2D(PhLine::*)(void)) &PhLine::GetPt2)
    ];


    /////////////////////////////
    // PhRect.hpp
    /////////////////////////////

    luabind::module(myLuaState)
    [
        luabind::class_<PhRect, PhysicObject, ProtectedPtr<Object> >("PhRect")
            .property("size", &PhRect:: GetSize, &PhRect:: SetSize)
            .def("GetSize",(Vector2D(PhRect::*)(void)) &PhRect::GetSize)
            .def("SetSize",(void(PhRect::*)(const Vector2D &)) &PhRect::SetSize)
            .property("sizeX", &PhRect:: GetSizeX, &PhRect:: SetSizeX)
            .property("sizeY", &PhRect:: GetSizeY, &PhRect:: SetSizeY)
            .def("GetSizeX",(float(PhRect::*)(void)) &PhRect::GetSizeX)
            .def("GetSizeY",(float(PhRect::*)(void)) &PhRect::GetSizeY)
            .def("SetSizeX",(void(PhRect::*)(float )) &PhRect::SetSizeX)
            .def("SetSizeY",(void(PhRect::*)(float )) &PhRect::SetSizeY)
    ];


    /////////////////////////////
    // PhShape.hpp
    /////////////////////////////

    luabind::module(myLuaState)
    [
        luabind::class_<PhShape, PhysicObject, ProtectedPtr<Object> >("PhShape")
            .def("GetPoint",(Vector2D(PhShape::*)(int )) &PhShape::GetPoint)
            .def("SetPoint",(void(PhShape::*)(int , const Vector2D &)) &PhShape::SetPoint)
            .def("AddPoint",(void(PhShape::*)(const Vector2D &)) &PhShape::AddPoint)
            .def("RemovePoint",(void(PhShape::*)(int )) &PhShape::RemovePoint)
    ];


    /////////////////////////////
    // PhTriangle.hpp
    /////////////////////////////

    luabind::module(myLuaState)
    [
        luabind::class_<PhTriangle, PhysicObject, ProtectedPtr<Object> >("PhTriangle")
            .def("GetPoint",(Vector2D(PhTriangle::*)(int )) &PhTriangle::GetPoint)
            .def("SetPoint",(void(PhTriangle::*)(int , const Vector2D &)) &PhTriangle::SetPoint)
    ];


    /////////////////////////////
    // PhysicWorld.hpp
    /////////////////////////////


    /////////////////////////////
    // PostFX.hpp
    /////////////////////////////

    luabind::module(myLuaState)
    [
        luabind::class_<PostFX, Drawable, ProtectedPtr<Object> >("PostFX")
            .def("LoadFromFile",(bool(PostFX::*)(const std::string& )) &PostFX::LoadFromFile)
            .def("SetParameter",(void(PostFX::*)(const std::string& , float )) &PostFX::SetParameter)
            .def("SetParameter",(void(PostFX::*)(const std::string& , float , float )) &PostFX::SetParameter)
            .def("SetParameter",(void(PostFX::*)(const std::string& , float , float , float )) &PostFX::SetParameter)
            .def("SetParameter",(void(PostFX::*)(const std::string& , float , float , float , float )) &PostFX::SetParameter)
            .def("SetTexture",(void(PostFX::*)(const std::string& , Texture* )) &PostFX::SetTexture)
            .def("CanUsePostFX",(bool(*)(void)) &PostFX::CanUsePostFX)
    ];


    /////////////////////////////
    // Shape.hpp
    /////////////////////////////

    luabind::module(myLuaState)
    [
        luabind::class_<Shape, Drawable, ProtectedPtr<Object> >("Shape")
            .property("color", &Shape:: GetColor, &Shape:: SetColor)
            .def("SetColor",(void(Shape::*)(const Color& )) &Shape::SetColor)
            .def("GetColor",(Color(Shape::*)(void)) &Shape::GetColor)
            .def("SetScale",(void(Shape::*)(const Vector2D& )) &Shape::SetScale)
            .def("GetScale",(Vector2D(Shape::*)(void)) &Shape::GetScale)
            .def("SetScaleX",(void(Shape::*)(float )) &Shape::SetScaleX)
            .def("SetScaleY",(void(Shape::*)(float )) &Shape::SetScaleY)
            .def("SetCenter",(void(Shape::*)(const Vector2D& )) &Shape::SetCenter)
            .def("GetCenter",(Vector2D(Shape::*)(void)) &Shape::GetCenter)
            .def("AddPoint",(void(Shape::*)(const Vector2D& , const Color& , const Color& )) &Shape::AddPoint)
            .def("GetNbPoints",(int(Shape::*)(void)) &Shape::GetNbPoints)
            .def("EnableFill",(void(Shape::*)(bool )) &Shape::EnableFill)
            .def("EnableOutline",(void(Shape::*)(bool )) &Shape::EnableOutline)
            .def("SetPointPosition",(void(Shape::*)(int , const Vector2D& )) &Shape::SetPointPosition)
            .def("SetPointColor",(void(Shape::*)(int , const Color& )) &Shape::SetPointColor)
            .def("SetPointOutlineColor",(void(Shape::*)(int , const Color& )) &Shape::SetPointOutlineColor)
            .def("SetOutlineWidth",(void(Shape::*)(float )) &Shape::SetOutlineWidth)
            .def("GetPointPosition",(Vector2D(Shape::*)(int )) &Shape::GetPointPosition)
            .def("GetPointColor",(Color(Shape::*)(int )) &Shape::GetPointColor)
            .def("GetPointOutlineColor",(Color(Shape::*)(int )) &Shape::GetPointOutlineColor)
            .def("GetOutlineWidth",(float(Shape::*)(void)) &Shape::GetOutlineWidth)
            .def("MakeLine",(void(Shape::*)(const Vector2D& , const Vector2D& , float , const Color& , float , const Color& )) &Shape::MakeLine)
            .def("MakeRectangle",(void(Shape::*)(const Vector2D& , const Vector2D& , const Color& , float , const Color& )) &Shape::MakeRectangle)
            .def("MakeCircle",(void(Shape::*)(const Vector2D& , float , const Color& , float , const Color& )) &Shape::MakeCircle)
            .def("SetSize",(void(Shape::*)(Vector2D &)) &Shape::SetSize)
    ];


    /////////////////////////////
    // Sprite.hpp
    /////////////////////////////

    luabind::module(myLuaState)
    [
        luabind::class_<Sprite, Drawable, ProtectedPtr<Object> >("Sprite")
            .property("size", &Sprite:: GetSize, &Sprite:: SetSize)
            .def("GetSize",(Vector2D(Sprite::*)(void)) &Sprite::GetSize)
            .def("SetSize",(void(Sprite::*)(const Vector2D &)) &Sprite::SetSize)
            .def("SetImage",(void(Sprite::*)(Texture &)) &Sprite::SetImage)
            .property("color", &Sprite:: GetColor, &Sprite:: SetColor)
            .def("SetColor",(void(Sprite::*)(const Color& )) &Sprite::SetColor)
            .def("GetColor",(Color(Sprite::*)(void)) &Sprite::GetColor)
            .def("GetScale",(Vector2D(Sprite::*)(void)) &Sprite::GetScale)
            .def("SetScaleX",(void(Sprite::*)(float )) &Sprite::SetScaleX)
            .def("SetScaleY",(void(Sprite::*)(float )) &Sprite::SetScaleY)
            .def("SetScale",(void(Sprite::*)(const Vector2D& )) &Sprite::SetScale)
            .def("SetCenter",(void(Sprite::*)(const Vector2D& )) &Sprite::SetCenter)
            .def("GetCenter",(Vector2D(Sprite::*)(void)) &Sprite::GetCenter)
            .def("FlipX",(void(Sprite::*)(bool )) &Sprite::FlipX)
            .def("FlipY",(void(Sprite::*)(bool )) &Sprite::FlipY)
            .def("GetPixel",(Color(Sprite::*)(unsigned int , unsigned int )) &Sprite::GetPixel)
    ];


    /////////////////////////////
    // Text.hpp
    /////////////////////////////

    luabind::module(myLuaState)
    [
        luabind::class_<Text, Drawable, ProtectedPtr<Object> >("Text")
            .property("text", &Text:: GetText, &Text:: SetText)
            .def("SetText",(void(Text::*)(const std::string& )) &Text::SetText)
            .def("GetText",(std::string(Text::*)(void)) &Text::GetText)
            .def("GetTextSize",(Vector2D(Text::*)(void)) &Text::GetTextSize)
            .property("maxWidth", &Text:: GetMaxWidth, &Text:: SetMaxWidth)
            .def("SetMaxWidth",(void(Text::*)(float )) &Text::SetMaxWidth)
            .def("GetMaxWidth",(float(Text::*)(void)) &Text::GetMaxWidth)
            .property("canCutWords", &Text:: IsCutWords, &Text:: SetCutWords)
            .def("SetCutWords",(void(Text::*)(bool )) &Text::SetCutWords)
            .def("IsCutWords",(bool(Text::*)(void)) &Text::IsCutWords)
            .property("color", &Text:: GetColor, &Text:: SetColor)
            .def("SetColor",(void(Text::*)(const Color& )) &Text::SetColor)
            .def("GetColor",(Color(Text::*)(void)) &Text::GetColor)
            .def("GetScale",(Vector2D(Text::*)(void)) &Text::GetScale)
            .def("SetScaleX",(void(Text::*)(float )) &Text::SetScaleX)
            .def("SetScaleY",(void(Text::*)(float )) &Text::SetScaleY)
            .def("SetScale",(void(Text::*)(const Vector2D& )) &Text::SetScale)
            .def("SetCenter",(void(Text::*)(const Vector2D& )) &Text::SetCenter)
            .def("GetCenter",(Vector2D(Text::*)(void)) &Text::GetCenter)
            .def("SetFont",(void(Text::*)(Font &)) &Text::SetFont)
            .def("GetFont",(ProtectedPtr<Font>(Text::*)(void)) &Text::GetFont)
            .def("SetSize",(void(Text::*)(float )) &Text::SetSize)
            .def("GetSize",(float(Text::*)(void)) &Text::GetSize)
            .def("SetFontSize",(void(Text::*)(float )) &Text::SetFontSize)
            .def("GetFontSize",(float(Text::*)(void)) &Text::GetFontSize)
            .property("bold", &Text:: SetBold, &Text:: GetBold)
            .def("SetBold",(void(Text::*)(bool )) &Text::SetBold)
            .def("GetBold",(bool(Text::*)(void)) &Text::GetBold)
            .property("italic", &Text:: SetItalic, &Text:: GetItalic)
            .def("SetItalic",(void(Text::*)(bool )) &Text::SetItalic)
            .def("GetItalic",(bool(Text::*)(void)) &Text::GetItalic)
            .property("underligned", &Text:: SetUnderligned, &Text:: GetUnderligned)
            .def("SetUnderligned",(void(Text::*)(bool )) &Text::SetUnderligned)
            .def("GetUnderligned",(bool(Text::*)(void)) &Text::GetUnderligned)
            .def("GetCharacterPos",(Vector2D(Text::*)(int )) &Text::GetCharacterPos)
    ];


    /////////////////////////////
    // ScreenRenderer.hpp
    /////////////////////////////

    luabind::module(myLuaState)
    [
        luabind::class_<ScreenRenderer, Drawable, ProtectedPtr<Object> >("ScreenRenderer")
            .property("targetScreen", &ScreenRenderer:: SetTargetScreen, &ScreenRenderer:: GetTargetScreen)
            .def("SetTargetScreen",(void(ScreenRenderer::*)(ProtectedPtr<Screen> )) &ScreenRenderer::SetTargetScreen)
            .def("GetTargetScreen",(ProtectedPtr<Screen>(ScreenRenderer::*)(void)) &ScreenRenderer::GetTargetScreen)
            .property("screenConfig", &ScreenRenderer:: SetScreenConfig, &ScreenRenderer:: GetScreenConfig)
            .def("GetScreenConfig",(ScreenConfig(ScreenRenderer::*)(void)) &ScreenRenderer::GetScreenConfig)
            .def("SetScreenConfig",(void(ScreenRenderer::*)(const ScreenConfig &)) &ScreenRenderer::SetScreenConfig)
            .property("isScreenConfigEnabled", &ScreenRenderer:: SetScreenConfig, &ScreenRenderer:: GetScreenConfig)
            .def("IsScreenConfigEnabled",(bool(ScreenRenderer::*)(void)) &ScreenRenderer::IsScreenConfigEnabled)
            .def("EnableScreenConfig",(void(ScreenRenderer::*)(bool )) &ScreenRenderer::EnableScreenConfig)
    ];


    /////////////////////////////
    // TexturedArea.hpp
    /////////////////////////////

    luabind::module(myLuaState)
    [
        luabind::class_<TexturedArea, Sprite, ProtectedPtr<Object> >("TexturedArea")
            .property("size", &TexturedArea:: GetAreaSize, &TexturedArea:: SetAreaSize)
            .def("GetAreaSize",(Vector2D(TexturedArea::*)(void)) &TexturedArea::GetAreaSize)
            .def("SetAreaSize",(void(TexturedArea::*)(const Vector2D &)) &TexturedArea::SetAreaSize)
            .property("textureSpeed", &TexturedArea:: SetTextureSpeed, &TexturedArea:: GetTextureSpeed)
            .def("SetTextureSpeed",(void(TexturedArea::*)(const Vector2D &)) &TexturedArea::SetTextureSpeed)
            .def("GetTextureSpeed",(Vector2D(TexturedArea::*)(void)) &TexturedArea::GetTextureSpeed)
            .property("textureOffset", &TexturedArea:: SetTextureOffset, &TexturedArea:: GetTextureOffset)
            .def("SetTextureOffset",(void(TexturedArea::*)(const Vector2D &)) &TexturedArea::SetTextureOffset)
            .def("GetTextureOffset",(Vector2D(TexturedArea::*)(void)) &TexturedArea::GetTextureOffset)
    ];


    /////////////////////////////
    // SoundManager.hpp
    /////////////////////////////

    luabind::module(myLuaState)
    [
        luabind::class_<SoundManager>("SoundManager")
            .def("PlaySound",(ProtectedPtr<SoundPlayer>(SoundManager::*)(Sound &)) &SoundManager::PlaySound)
            .def("StopAllSounds",(void(SoundManager::*)(void)) &SoundManager::StopAllSounds)
    ];

    luabind::module(myLuaState)
    [
        luabind::class_<SoundPlayer, ProtectedPtr<SoundPlayer> >("SoundPlayer")
            .def("Stop",(void(SoundPlayer::*)(void)) &SoundPlayer::Stop)
            .def("SetSound",(void(SoundPlayer::*)(En::Sound *)) &SoundPlayer::SetSound)
            .def("SetLoop",(void(SoundPlayer::*)(bool )) &SoundPlayer::SetLoop)
            .def("SetPitch",(void(SoundPlayer::*)(float )) &SoundPlayer::SetPitch)
            .def("SetVolume",(void(SoundPlayer::*)(float )) &SoundPlayer::SetVolume)
            .def("SetPosition",(void(SoundPlayer::*)(float , float , float )) &SoundPlayer::SetPosition)
            .def("SetPosition",(void(SoundPlayer::*)(const Vector3D& )) &SoundPlayer::SetPosition)
            .def("SetMinDistance",(void(SoundPlayer::*)(float )) &SoundPlayer::SetMinDistance)
            .def("SetAttenuation",(void(SoundPlayer::*)(float )) &SoundPlayer::SetAttenuation)
            .def("SetPlayingOffset",(void(SoundPlayer::*)(float )) &SoundPlayer::SetPlayingOffset)
            .def("GetLoop",(bool(SoundPlayer::*)(void)) &SoundPlayer::GetLoop)
            .def("GetPitch",(float(SoundPlayer::*)(void)) &SoundPlayer::GetPitch)
            .def("GetVolume",(float(SoundPlayer::*)(void)) &SoundPlayer::GetVolume)
            .def("GetPosition",(Vector3D(SoundPlayer::*)(void)) &SoundPlayer::GetPosition)
            .def("IsRelativeToListener",(bool(SoundPlayer::*)(void)) &SoundPlayer::IsRelativeToListener)
            .def("GetMinDistance",(float(SoundPlayer::*)(void)) &SoundPlayer::GetMinDistance)
            .def("GetAttenuation",(float(SoundPlayer::*)(void)) &SoundPlayer::GetAttenuation)
            .def("GetPlayingOffset",(float(SoundPlayer::*)(void)) &SoundPlayer::GetPlayingOffset)
    ];

}
//------------------------------------------------------------------------------
} // namespace En
//------------------------------------------------------------------------------
