//------------------------------------------------------------------------------
/*
    This file is part of Bottle2D.

    Bottle2D is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Bottle2D is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Bottle2D.  If not, see <http://www.gnu.org/licenses/>.
*/
//------------------------------------------------------------------------------
#include "PostFX.hpp"
//------------------------------------------------------------------------------
namespace En
{
//------------------------------------------------------------------------------
PostFX::PostFX(Group *parent, const std::string &name) : DrawableSF(parent, name)
{
    //ctor
}
//------------------------------------------------------------------------------
bool PostFX::LoadFromFile(const std::string& Filename)
{
    bool success = sf::PostFX::LoadFromFile(Filename);
    SetTexture("framebuffer", NULL);

    return success;
}
//------------------------------------------------------------------------------
void PostFX::Render(sf::RenderTarget &App)
{
    // Check for invalidated pointers
    if(!AreResourcesOk())
    {
        MY_THROW("PostFX::Render(): the texture used by the sprite was deleted ! Object name: '" << GetName() << "'.")
    }

    // Render
    SetGlobalPosition(Vector2D(0, 0));
    RenderSf(App, this);
}
//------------------------------------------------------------------------------
void PostFX::SetTexture(const std::string& name, Texture* image)
{
    if(!((image == NULL) && (name == "framebuffer"))) // null = framebuffer => ok
    {
        ProtectedPtr<ProtectedClassBase> ptr(image);

        if(!ptr.IsValid())
        {
            MY_THROW("PostFX::SetTexture(): the texture was deleted ! Object name: '" << GetName() << "'.");
        }

        // Ressources
        AddResource(ptr);
    }

    // Set the texture
    sf::PostFX::SetTexture(name, image);
}
//------------------------------------------------------------------------------
} // namespace En
//------------------------------------------------------------------------------
