//------------------------------------------------------------------------------
/*
    This file is part of Bottle2D.

    Bottle2D is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Bottle2D is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Bottle2D.  If not, see <http://www.gnu.org/licenses/>.
*/
//------------------------------------------------------------------------------
#ifndef SCREENRENDERER_HPP
#define SCREENRENDERER_HPP
//------------------------------------------------------------------------------
#include "Drawable.hpp"
#include "Screen.hpp"
//------------------------------------------------------------------------------
namespace En
{
//------------------------------------------------------------------------------
class Screen;
//------------------------------------------------------------------------------
/// LUA_HEADER_COPY_ON ///
/** A screen renderer is used to draw the content of an other screen in the current screen. It can be usefull if you want to draw the content of your game in the menu background, for example.
  * This object is created using Group::CreateScreenRenderer.
  */
/// LUA_HEADER_COPY_OFF ///
class ScreenRenderer : public Drawable
{
public:
    ScreenRenderer(Group *parent, const std::string &name);

    /// LUA_EXPORT_ON(ScreenRenderer, Drawable, ProtectedPtr<Object>) ///

    // Screen
    /** Access drawed screen */
    //LUA// luaProperty(Screen, targetScreen, SetTargetScreen, GetTargetScreen);

    /** Set the target screen */
    void SetTargetScreen(ProtectedPtr<Screen> targetScreen);

    /** Get the target screen */
    ProtectedPtr<Screen> GetTargetScreen();

    // Screen config
    /** Access target screen config */
    //LUA// luaProperty(ScreenConfig, screenConfig, SetScreenConfig, GetScreenConfig);

    /** Get the target screen config : this screen config will be used to render the target screen */
    ScreenConfig GetScreenConfig() const;

    /** Set the target screen config and enable it : this screen config will be used to render the target screen */
    void SetScreenConfig(const ScreenConfig &screenConfig);

    // Enable / disable screen config
    /** True if the target screen use a special screenConfig (set with SetScreenConfig()), false otherwise. */
    //LUA// luaProperty(bool, isScreenConfigEnabled, SetScreenConfig, GetScreenConfig);

    /** Return true if the target screen use a special screenConfig (set with SetScreenConfig()), false otherwise. */
    bool IsScreenConfigEnabled() const;

    /** Enable or disable the screenConfig. If it is diabled, the screen will be rendered with its own screen config. */
    void EnableScreenConfig(bool isScreenConfigEnabled);

    /// LUA_EXPORT_OFF ///


    /** Create a clone of the object, change the parent and return an handler to the clone. */
    virtual ScreenRenderer *GetRawCloneToGroup(Group *newParent) const {ScreenRenderer *obj = new ScreenRenderer(*this); obj->RegisterClone(newParent); return obj;}

    // Type
    ADD_GET_TYPE_FN(OBJ_TYPE_DRAWABLE_SCREEN_RENDERER)

    // Render
    virtual void Render(sf::RenderTarget &App);

protected:
    ProtectedPtr<Screen> m_targetScreen;
    ScreenConfig m_screenConfig;
    bool m_isScreenConfigEnabled;
};
//----------------------------------------------------------------------------
} // namespace En

//------------------------------------------------------------------------------
#endif // SCREENRENDERER_HPP
