
------------------------------------------------------------
-- Checkpoint load / save
------------------------------------------------------------

function createCheckpoint()	
	local objTbl = levelGrp:GetUserData("checkpointObjTbl")
	local statusTbl = {}
	
	for i = 1, #objTbl do
		statusTbl[i] = objTbl[i]:GetUserData("getStatusFn")(objTbl[i])
	end
	
	levelGrp:SetUserData("checkpointStatusTbl", statusTbl)
end

function restoreCheckpoint()
	-- We save the status
	local statusTbl = levelGrp:GetUserData("checkpointStatusTbl")

	-- We reload the level
	-- /!\ levelGrp is deleted and recreated !
	initLevelObjs()
	
	-- We apply the status
	local objTbl = levelGrp:GetUserData("checkpointObjTbl")
	
	for i = 1, #objTbl do
		objTbl[i]:GetUserData("setStatusFn")(objTbl[i], statusTbl[i])
	end
	
	-- We put the status to the levelGrp again, in order to reload it if needed
	levelGrp:SetUserData("checkpointStatusTbl", statusTbl)
end

function registerSavedCheckpointObject(grp)
	if not levelGrp:UserDataExists("checkpointObjTbl") then
		levelGrp:SetUserData("checkpointObjTbl", {})
	end

	local objTbl = levelGrp:GetUserData("checkpointObjTbl")
	table.insert(objTbl, grp)
end

------------------------------------------------------------
-- Checkpoint zone
------------------------------------------------------------

function createCheckpointZone(parentGrp, position, size)
	local grp = parentGrp:CreateGroup("checkpointZoneGrp", position)
	grp:SetUserData("isAlreadyCreated", false) -- We create only one checkpoint per zone !

	-- Ph
	local ph = grp:CreatePhRect(phGroupButton, "ph", Vector2D(0, 0), size)
	
	-- Fn
	initButtonFn(ph, function(player, button)
						local grp = button:GetParent()

						local isAlreadyCreated = grp:GetUserData("isAlreadyCreated")
						if not isAlreadyCreated then
							grp:SetUserData("isAlreadyCreated", true)
							createCheckpoint()
							
							-- Notification
							createInfoMsg(gameRenderScreen, "Point de passage atteint.", 2)
							soundNs.PlaySnd(sndCheckpoint, gameScreen)
						end
					end, nil)

	-- Checkpoint
	grp:SetUserData("getStatusFn", function(grp) 
										return grp:GetUserData("isAlreadyCreated")
									end)
	grp:SetUserData("setStatusFn", function(grp, status)
										grp:SetUserData("isAlreadyCreated", status)
									end)
	registerSavedCheckpointObject(grp)
end
