
------------------------------------------------------------
-- Key events
------------------------------------------------------------

function onPlayerLaunch(keyName, pressedStatus, event)
	local player = event:GetUserData("player")
	
	if player:GetUserData("mode") == "throw" then -- Mode lancée
		direction, speed = getSightInfo(player)
		createMovingBombe(player:GetParent(), direction, speed, player)
		playerSetMode(player, "move") -- Plus en mode visée
		
		-- Image lancée
		animChoseOneImage(player:FindByNameSprite("spr"), 3)
		
		-- Stats
		levelData.stats.minNbLaunch = levelData.stats.minNbLaunch + 1
	elseif player:GetUserData("mode") == "look" then -- Mode lancée
		playerSetMode(player, "throw")
	end
end

function onJump(keyName, pressedStatus, event)
	local player = event:GetUserData("player")
	
	
	if player:GetUserData("mode") == "move" then -- Mode marche
		-- Si on est déjà dans les airs
		if player:GetUserData("isOnAir") == false then
			startGravity(player, true)
			player:SetSpeed(Vector2D(player:GetSpeed().x, -jumpSpeed))
			
			-- Snd
			soundNs.PlaySnd(sndPlayerJump, gameScreen)
		end
	elseif player:GetUserData("mode") == "throw" then -- Mode lancée
		playerSetMode(player, "look") -- On se remet en look
	end
end

function onFramePlayerMove(frameTime, event)
	local walk = false

	local player = event:GetUserData("player")
	local direction = Vector2D(0, 0)
	
	if IsKeyPressed(player:GetUserData("keyTbl")["keyRight"]) then
		direction = Vector2D(1, 0)
	elseif IsKeyPressed(player:GetUserData("keyTbl")["keyLeft"]) then
		direction = Vector2D(-1, 0)
	end
	
	local currentMode = player:GetUserData("mode")
	
	-- If the frame is different, the player become white (no color) again
	if isLevelChallenge() and (player:GetUserData("frameIdColor") ~= GetUniqueFrameId()) then
		player:FindByName("spr"):SetColor(Color(255, 255, 255))
	end
	
	-- Walk
	if direction ~= Vector2D(0, 0) then -- On a choisi une direction		
		if currentMode == "move" then -- Mode marche
			
			startGravity(player, true)
			
			if direction.y == 0 then
				if player:GetUserData("isOnAir") == true then
					local newSpeed = player:GetSpeed().x + direction.x * onAirAcceleration * frameTime
					if newSpeed > 0 then
						newSpeed = math.min(newSpeed, onAirMaxXSpeed)
					else
						newSpeed = math.max(newSpeed, - onAirMaxXSpeed)
					end
										
					player:SetSpeed(Vector2D(newSpeed, player:GetSpeed().y))
				else
					local newSpeedX = player:GetSpeed().x + walkSpeedAcceleration * frameTime * direction.x
					if newSpeedX > walkSpeedX then
						newSpeedX = walkSpeedX
					elseif newSpeedX < -walkSpeedX then
						newSpeedX = -walkSpeedX
					end

					player:SetSpeed(Vector2D(newSpeedX, player:GetSpeed().y))

					walk = true
				end
			end
		elseif currentMode == "throw" then -- Mode lancée
			-- On le remet en mode vue si il touche aux flèches
			playerSetMode(player, "look")
		end
	end
	
	if currentMode == "look" then
		-- Navigation
		
		local lookDecal = player:GetUserData("lookDecal")
		lookDecal = lookDecal + direction * frameTime * decalSpeed;
		
		direction = Vector2D(0, 0)
		if IsKeyPressed(player:GetUserData("keyTbl")["keyJump"]) then
			direction = Vector2D(0, -1)
		elseif IsKeyPressed(player:GetUserData("keyTbl")["keyAction"]) then
			direction = Vector2D(0, 1)
		end
		
		lookDecal = lookDecal + direction * frameTime * decalSpeed;
		player:SetUserData("lookDecal", lookDecal)
		onPlayerMove(player)
	end
	
	-- Show player walk
	viewPlayerWalk(walk, player, direction.x)
end

------------------------------------------------------------
-- Set mode
------------------------------------------------------------

function playerSetMode(playerGrp, newMode)
	if not playerGrp:UserDataExists("mode") then
		printErrorConsole("Error: no mode data set in player !")
	else
		if (newMode == "move") or (newMode == "throw") or (newMode == "none") or (newMode == "look") then
			playerGrp:SetUserData("mode", newMode)

			if newMode == "throw" then
				beginSight(playerGrp)
			else
				endSight(playerGrp)
				clearBombeTrajectoire()
			end
			
			playerGrp:SetUserData("lookDecal", Vector2D(0, 0)) -- On remet le décallage à 0
			if newMode == "throw" then -- Quand on se met en mode lancé, la camera se remet en place avec un anim
				viewCameraReturnToPlayer(playerGrp)
			else
				onPlayerMove(playerGrp)
			end
		else
			printErrorConsole("Error: mode unknow in player !")
		end
		
		viewPlayerChangeMode(playerGrp, newMode)
	end
end

function onPlayerAction(keyName, pressedStatus, event)
	local player = event:GetUserData("player")
	
	
	if player:GetUserData("mode") == "move" then -- Mode marche
		if UserDataExistsAndIsNotEqual(player, "_currentButton", nil) then
			bt = player:GetUserData("_currentButton")
			if bt:IsIntersect(player:FindByNamePhRect("body")) then -- Si on est toujours dans le bouton
				if UserDataExistsAndIsNotEqual(bt, "_btFnOnAction", nil) then
					bt:GetUserData("_btFnOnAction")(player, bt)
				end
			end
		end
	elseif player:GetUserData("mode") == "throw" then -- Mode lancée
		playerSetMode(player, "look")
	end
end

function initPlayer(playerId, parentGroup, keyTbl, pos)
	-- Player
	local playerGrp = parentGroup:CreateGroup("playerGrp")
	playerGrp:SetPosition(pos)
	playerGrp:SetUserData("playerId", playerId)
	playerGrp:SetUserData("isOnAir", true)
	playerGrp:SetUserData("keyTbl", keyTbl)
	playerGrp:SetUserData("mode", "none")
	playerGrp:SetUserData("lookDecal", Vector2D(0, 0))
	playerGrp:SetUserData("frameIdColor", -1)
	startGravity(playerGrp, true)
	
	-- Register
	table.insert(levelPlayerTbl, playerGrp)
	
	-- Physic
	local ph = playerGrp:CreatePhRect(phGroupPlayer, "body", Vector2D(playerSize.x, playerSize.y))
	ph:SetEventFunction(onPlayerCorpsColision)
	
	-- Design
	local spr = playerGrp:CreateSprite("spr")	
	animCreate(spr, playerAnimTbl[playerId], 0.2)
	animChoseOneImage(spr, 5)
	spr:SetSize(playerSize)
		
	-- Events
	local event = parentGroup:GetParentScreen():RegisterFrameEvent(onFramePlayerMove)
	guiLib_addAutoDestroyEvent(playerGrp, event)
	event:SetUserData("player", playerGrp)
	
	local event = parentGroup:GetParentScreen():RegisterButtonEvent(onJump, keyTbl["keyJump"], true)
	guiLib_addAutoDestroyEvent(playerGrp, event)
	event:SetUserData("player", playerGrp)
	
	local event = parentGroup:GetParentScreen():RegisterButtonEvent(onPlayerAction, keyTbl["keyAction"], true)
	guiLib_addAutoDestroyEvent(playerGrp, event)
	event:SetUserData("player", playerGrp)
	
	local event = parentGroup:GetParentScreen():RegisterButtonEvent(onPlayerLaunch, "MouseLeft", true)
	guiLib_addAutoDestroyEvent(playerGrp, event)
	event:SetUserData("player", playerGrp)
	
	-- Fake mouse
	if userCfg.keyConf.isFakeMouse then
		local event = parentGroup:GetParentScreen():RegisterButtonEvent(onPlayerLaunch, userCfg.keyConf.fakeMouseKeys["keyLaunch"], true)
		guiLib_addAutoDestroyEvent(playerGrp, event)
		event:SetUserData("player", playerGrp)
	end
	
	-- Init view
	onPlayerMove(playerGrp)
	
	-- Checkpoint
	playerGrp:SetUserData("getStatusFn", function(grp)
											local status = {}
											status.mode = grp:GetUserData("mode")
											status.position = grp:GetPosition()
											status.speed = grp:GetSpeed()
											status.lookDecal = grp:GetUserData("lookDecal")
											return status
										end)
	playerGrp:SetUserData("setStatusFn", function(grp, status)
											grp:SetPosition(status.position)
											grp:SetSpeed(status.speed)
											playerSetMode(grp, status.mode)
											grp:SetUserData("lookDecal", status.lookDecal)
											adaptZoom({grp})
										end)
	registerSavedCheckpointObject(playerGrp)
	
	-- Zpos
	playerGrp:SetZpos(40)
	
	-- Return
	return playerGrp
end

